//
// hello.mag.45.c
//
// Hall effect magnetic sensing hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 11/3/13
// (c) Massachusetts Institute of Technology 2013
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 100 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTB
#define serial_direction DDRB
#define serial_pin_out (1 << PB2)
#define led_pin_out (1 << PB0)
#define button_pin_in (1 << PB1)

#define nsamples 64 // number of samples to accumulate

#define EEPROM_DEF 0xFF
#define thresh_default 25600 // 400*64, 500 is normal magnetic field

/* Wort */
uint16_t eeThresh EEMEM;

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

int main(void) {
   //
   // main
   //
   static uint16_t count;
   static uint16_t but_count;
   static uint16_t i;
   static uint16_t accum;
   static uint16_t thresh;
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);
   //
   // init A/D
   //
   ADMUX = (0 << REFS2) | (0 << REFS1) | (0 << REFS0) // Vcc ref
      | (0 << ADLAR) // right adjust
      | (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (0 << MUX0); // ADC4
   ADCSRA = (1 << ADEN) // enable
      | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128

   // init LED and Switch
   DDRB |= (1<<PB0);		// LED ist OUT an PB0
   PORTB |= (1<<PB1);		// Switch Vorwiderstand an PB1

   // load thresh from the EEPROM
   if ((thresh = eeprom_read_word (&eeThresh)) == EEPROM_DEF)
    {
        thresh = thresh_default;
    }
   //
   // main loop
   //
   while (1) {
      //
      // accumulate samples
      //
      accum = 0;
      for (count = 0; count < nsamples; ++count) { 
         //
         // initiate conversion
         //
         ADCSRA |= (1 << ADSC);
         //
         // wait for completion
         //
         while (ADCSRA & (1 << ADSC))
            ;
         //
         // add result
         //
         accum += ADC;
         }

	//clear(PORTB, led_pin_out);
      // check if button pressed
/*
      if ( ~ pin_test(PINB, button_pin_in) )
         set(PORTB, led_pin_out);
      else
         clear(PORTB, led_pin_out);
*/



      but_count=0;
      while ((PINB & (1<<PB1)) == 0 && but_count < 30) { 
          but_count++;
          _delay_us(100000);
      } 
      if ( but_count >= 30 ) {
	// entering programming mode
	thresh = accum;
	eeprom_write_word (&eeThresh, thresh);
	for ( i=0; i<10; i++) {
         PORTB |= (1<<PB0);
          _delay_us(100000);
          PORTB &= ~ (1<<PB0);
          _delay_us(100000);
        }
      } else {
	if (accum < thresh){
         PORTB |= (1<<PB0);
	} else {
          PORTB &= ~ (1<<PB0);
	}
      }

      //
      // send framing
      //
      put_char(&serial_port, serial_pin_out, 1);
      char_delay();
      put_char(&serial_port, serial_pin_out, 2);
      char_delay();
      put_char(&serial_port, serial_pin_out, 3);
      char_delay();
      put_char(&serial_port, serial_pin_out, 4);
      char_delay();
      //
      // send result
      //
      put_char(&serial_port, serial_pin_out, (accum & 255));
      char_delay();
      put_char(&serial_port, serial_pin_out, ((accum >> 8) & 255));
      char_delay();
      put_char(&serial_port, serial_pin_out, ((accum >> 16) & 255));
      char_delay();
      }
   }
